@@ -2,9 +2,8 @@
 # this repository contains the full copyright notices and license terms.
 
 from trytond.pool import Pool
-from . import product
-from . import account
-from . import stock
+
+from . import account, product, stock
 
 
 def register():
@@ -17,6 +16,12 @@ def register():
         account.LandedCost_Shipment,
         account.LandedCost_ProductCategory,
         account.LandedCost_Product,
+        account.LandedCostShow,
+        account.LandedCostShowMove,
         account.InvoiceLine,
         stock.Move,
         module='account_stock_landed_cost', type_='model')
+    Pool.register(
+        account.PostLandedCost,
+        account.ShowLandedCost,
+        module='account_stock_landed_cost', type_='wizard')
