@@ -1,19 +1,22 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from decimal import Decimal, ROUND_DOWN, ROUND_HALF_EVEN
+from decimal import ROUND_DOWN, ROUND_HALF_EVEN, Decimal
+from itertools import groupby
 from operator import itemgetter
 
+from sql.functions import CharLength
+
 from trytond.i18n import gettext
-from trytond.model import ModelSQL, ModelView, Workflow, MatchMixin, fields
+from trytond.model import (
+    Index, MatchMixin, ModelSQL, ModelView, Workflow, fields)
+from trytond.modules.company.model import CompanyValueMixin
+from trytond.modules.product import price_digits, round_price
 from trytond.pool import Pool, PoolMeta
 from trytond.pyson import Eval, Id
 from trytond.transaction import Transaction
-from trytond import backend
-from trytond.tools.multivalue import migrate_property
-from trytond.modules.company.model import CompanyValueMixin
-from trytond.modules.product import round_price
+from trytond.wizard import Button, StateTransition, StateView, Wizard
 
-from .exceptions import NoMoveWarning, FilterUnusedWarning
+from .exceptions import FilterUnusedWarning, NoMoveWarning
 
 
 def _parents(records):
@@ -50,26 +53,7 @@ class ConfigurationLandedCostSequence(ModelSQL, CompanyValueMixin):
             ('company', 'in', [Eval('company', -1), None]),
             ('sequence_type', '=',
                 Id('account_stock_landed_cost', 'sequence_type_landed_cost')),
-            ],
-        depends=['company'])
-
-    @classmethod
-    def __register__(cls, module_name):
-        exist = backend.TableHandler.table_exist(cls._table)
-
-        super(ConfigurationLandedCostSequence, cls).__register__(module_name)
-
-        if not exist:
-            cls._migrate_property([], [], [])
-
-    @classmethod
-    def _migrate_property(cls, field_names, value_names, fields):
-        field_names.append('landed_cost_sequence')
-        value_names.append('landed_cost_sequence')
-        fields.append('company')
-        migrate_property(
-            'account.configuration', field_names, cls, value_names,
-            fields=fields)
+            ])
 
     @classmethod
     def default_landed_cost_sequence(cls, **pattern):
@@ -86,12 +70,11 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
     'Landed Cost'
     __name__ = 'account.landed_cost'
     _rec_name = 'number'
-    number = fields.Char('Number', select=True, readonly=True)
+    number = fields.Char("Number", readonly=True)
     company = fields.Many2One('company.company', 'Company', required=True,
         states={
             'readonly': Eval('state') != 'draft',
-            },
-        depends=['state'])
+            })
     shipments = fields.Many2Many('account.landed_cost-stock.shipment.in',
         'landed_cost', 'shipment', 'Shipments',
         states={
@@ -100,8 +83,7 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
         domain=[
             ('company', '=', Eval('company')),
             ('state', 'in', ['received', 'done']),
-            ],
-        depends=['company', 'state'])
+            ])
     invoice_lines = fields.One2Many('account.invoice.line', 'landed_cost',
         'Invoice Lines',
         states={
@@ -114,17 +96,15 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
             ('company', '=', Eval('company', -1)),
             ('invoice.state', 'in', ['posted', 'paid']),
             ('invoice.type', '=', 'in'),
-            ('product.template.landed_cost', '=', True),
+            ('product.landed_cost', '=', True),
             ('type', '=', 'line'),
-            ],
-        depends=['state', 'company'])
+            ])
     allocation_method = fields.Selection([
             ('value', 'By Value'),
             ], 'Allocation Method', required=True,
         states={
             'readonly': Eval('state') != 'draft',
-            },
-        depends=['state'])
+            })
 
     categories = fields.Many2Many(
         'account.landed_cost-product.category', 'landed_cost', 'category',
@@ -132,7 +112,6 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
         states={
             'readonly': Eval('state') != 'draft',
             },
-        depends=['state'],
         help="Apply only to products of these categories.")
     products = fields.Many2Many(
         'account.landed_cost-product.product', 'landed_cost', 'product',
@@ -140,7 +119,6 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
         states={
             'readonly': Eval('state') != 'draft',
             },
-        depends=['state'],
         help="Apply only to these products.")
 
     posted_date = fields.Date('Posted Date', readonly=True)
@@ -148,12 +126,17 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
             ('draft', 'Draft'),
             ('posted', 'Posted'),
             ('cancelled', 'Cancelled'),
-            ], 'State', readonly=True)
+            ], "State", readonly=True, sort=False)
+
+    factors = fields.Dict(None, "Factors", readonly=True)
 
     @classmethod
     def __setup__(cls):
+        cls.number.search_unaccented = False
         super(LandedCost, cls).__setup__()
-
+        t = cls.__table__()
+        cls._sql_indexes.add(
+            Index(t, (t.state, Index.Equality()), where=t.state == 'draft'))
         cls._order = [
             ('number', 'DESC'),
             ('id', 'DESC'),
@@ -173,22 +156,21 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
                     'invisible': Eval('state') != 'cancelled',
                     'depends': ['state'],
                     },
-                'post': {
+                'post_wizard': {
                     'invisible': Eval('state') != 'draft',
                     'depends': ['state'],
                     },
+                'show': {
+                    'invisible': Eval('state').in_(['draft', 'cancelled']),
+                    'depends': ['state']
+                    },
                 })
 
     @classmethod
     def __register__(cls, module_name):
         cursor = Transaction().connection.cursor()
-        table_h = cls.__table_handler__(module_name)
         sql_table = cls.__table__()
 
-        # Migration from 3.8: rename code into number
-        if table_h.column_exist('code'):
-            table_h.column_rename('code', 'number')
-
         super(LandedCost, cls).__register__(module_name)
 
         # Migration from 5.6: rename state cancel to cancelled
@@ -196,6 +178,11 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
                 [sql_table.state], ['cancelled'],
                 where=sql_table.state == 'cancel'))
 
+    @classmethod
+    def order_number(cls, tables):
+        table, _ = tables[None]
+        return [CharLength(table.number), table.number]
+
     @staticmethod
     def default_company():
         return Transaction().context.get('company')
@@ -218,6 +205,7 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
                     landed_cost.allocation_method)()
         cls.write(landed_costs, {
                 'posted_date': None,
+                'factors': None,
                 'state': 'cancelled',
                 })
 
@@ -293,10 +281,17 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
                             product=product.rec_name))
 
     def allocate_cost_by_value(self):
-        self._allocate_cost(self._get_value_factors())
+        self.factors = self._get_factors('value')
+        self._allocate_cost(self.factors)
 
     def unallocate_cost_by_value(self):
-        self._allocate_cost(self._get_value_factors(), sign=-1)
+        factors = self.factors or self._get_factors('value')
+        self._allocate_cost(factors, sign=-1)
+
+    def _get_factors(self, method=None):
+        if method is None:
+            method = self.allocation_method
+        return getattr(self, '_get_%s_factors' % method)()
 
     def _get_value_factors(self):
         "Return the factor for each move based on value"
@@ -320,29 +315,23 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
         for move in moves:
             quantity = Decimal(str(move.quantity))
             if not sum_value:
-                factors[move.id] = 1 / length
+                factors[str(move.id)] = 1 / length
             else:
-                factors[move.id] = quantity * unit_prices[move.id] / sum_value
+                factors[str(move.id)] = (
+                    quantity * unit_prices[move.id] / sum_value)
         return factors
 
-    def _allocate_cost(self, factors, sign=1):
-        "Allocate cost on moves using factors"
+    def _costs_to_allocate(self, moves, factors):
         pool = Pool()
         Move = pool.get('stock.move')
-        Currency = pool.get('currency.currency')
-        assert sign in {1, -1}
-
         cost = self.cost
-        currency = self.company.currency
-        moves = [m for m in self.stock_moves() if m.quantity]
-
         costs = []
         digit = Move.unit_price.digits[1]
         exp = Decimal(str(10.0 ** -digit))
         difference = cost
         for move in moves:
             quantity = Decimal(str(move.quantity))
-            move_cost = cost * factors[move.id]
+            move_cost = cost * factors[str(move.id)]
             unit_landed_cost = round_price(
                 move_cost / quantity, rounding=ROUND_DOWN)
             costs.append({
@@ -360,6 +349,18 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
                 difference -= exp * quantity
             if difference < exp:
                 break
+        return costs
+
+    def _allocate_cost(self, factors, sign=1):
+        "Allocate cost on moves using factors"
+        pool = Pool()
+        Move = pool.get('stock.move')
+        Currency = pool.get('currency.currency')
+        assert sign in {1, -1}
+
+        currency = self.company.currency
+        moves = [m for m in self.stock_moves() if m.quantity]
+        costs = self._costs_to_allocate(moves, factors)
 
         for cost in costs:
             move = cost['move']
@@ -376,12 +377,24 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
         Move.save(moves)
 
     @classmethod
-    @ModelView.button
+    @ModelView.button_action(
+        'account_stock_landed_cost.wizard_landed_cost_post')
+    def post_wizard(cls, landed_costs):
+        pass
+
+    @classmethod
+    @ModelView.button_action(
+        'account_stock_landed_cost.wizard_landed_cost_show')
+    def show(cls, landed_costs):
+        pass
+
+    @classmethod
     @Workflow.transition('posted')
     def post(cls, landed_costs):
         pool = Pool()
         Date = pool.get('ir.date')
         Warning = pool.get('res.user.warning')
+        today = Date.today()
 
         for landed_cost in landed_costs:
             stock_moves = landed_cost.stock_moves()
@@ -396,9 +409,15 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
             landed_cost._stock_move_filter_unused(stock_moves)
             getattr(landed_cost, 'allocate_cost_by_%s' %
                 landed_cost.allocation_method)()
-        cls.write(landed_costs, {
-                'posted_date': Date.today(),
-                })
+        for company, c_landed_costs in groupby(
+                landed_costs, key=lambda l: l.company):
+            with Transaction().set_context(company=company.id):
+                today = Date.today()
+            for landed_cost in c_landed_costs:
+                landed_cost.posted_date = today
+            landed_cost.posted_date = today
+        # Use save as allocate methods may modify the records
+        cls.save(landed_costs)
 
     @classmethod
     def create(cls, vlist):
@@ -425,34 +444,114 @@ class LandedCost(Workflow, ModelSQL, ModelView, MatchMixin):
 class LandedCost_Shipment(ModelSQL):
     'Landed Cost - Shipment'
     __name__ = 'account.landed_cost-stock.shipment.in'
-    landed_cost = fields.Many2One('account.landed_cost', 'Landed Cost',
-        required=True, select=True)
-    shipment = fields.Many2One('stock.shipment.in', 'Shipment',
-        required=True)
+    landed_cost = fields.Many2One(
+        'account.landed_cost', 'Landed Cost',
+        required=True, ondelete='CASCADE')
+    shipment = fields.Many2One(
+        'stock.shipment.in', 'Shipment', required=True, ondelete='CASCADE')
 
 
 class LandedCost_ProductCategory(ModelSQL):
     "Landed Cost - Product Category"
     __name__ = 'account.landed_cost-product.category'
     landed_cost = fields.Many2One(
-        'account.landed_cost', 'Landed Cost', required=True, select=True)
+        'account.landed_cost', 'Landed Cost',
+        required=True, ondelete='CASCADE')
     category = fields.Many2One(
-        'product.category', "Category", required=True)
+        'product.category', "Category", required=True, ondelete='CASCADE')
 
 
 class LandedCost_Product(ModelSQL):
     "Landed Cost - Product"
     __name__ = 'account.landed_cost-product.product'
     landed_cost = fields.Many2One(
-        'account.landed_cost', "Landed Cost", required=True, select=True)
+        'account.landed_cost', "Landed Cost",
+        required=True, ondelete='CASCADE')
     product = fields.Many2One(
-        'product.product', "Product", required=True)
+        'product.product', "Product", required=True, ondelete='CASCADE')
+
+
+class ShowLandedCostMixin(Wizard):
+    start_state = 'show'
+    show = StateView('account.landed_cost.show',
+        'account_stock_landed_cost.landed_cost_show_view_form', [])
+
+    @property
+    def factors(self):
+        return self.record._get_factors()
+
+    def default_show(self, fields):
+        moves = []
+        default = {
+            'cost': round_price(self.record.cost),
+            'moves': moves,
+            }
+        stock_moves = [m for m in self.record.stock_moves() if m.quantity]
+        costs = self.record._costs_to_allocate(stock_moves, self.factors)
+        for cost in costs:
+            moves.append({
+                    'move': cost['move'].id,
+                    'cost': round_price(
+                        cost['unit_landed_cost'], rounding=ROUND_HALF_EVEN),
+                    })
+        return default
+
+
+class PostLandedCost(ShowLandedCostMixin):
+    "Post Landed Cost"
+    __name__ = 'account.landed_cost.post'
+    post = StateTransition()
+
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.show.buttons.extend([
+                Button("Cancel", 'end', 'tryton-cancel'),
+                Button("Post", 'post', 'tryton-ok', default=True),
+                ])
+
+    def transition_post(self):
+        self.model.post([self.record])
+        return 'end'
+
+
+class ShowLandedCost(ShowLandedCostMixin):
+    "Show Landed Cost"
+    __name__ = 'account.landed_cost.show'
+
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.show.buttons.extend([
+                Button("Close", 'end', 'tryton-close', default=True),
+                ])
+
+    @property
+    def factors(self):
+        return self.record.factors or super().factors
+
+
+class LandedCostShow(ModelView):
+    "Landed Cost Show"
+    __name__ = 'account.landed_cost.show'
+
+    cost = fields.Numeric("Cost", digits=price_digits, readonly=True)
+    moves = fields.One2Many(
+        'account.landed_cost.show.move', None, "Moves", readonly=True)
+
+
+class LandedCostShowMove(ModelView):
+    "Landed Cost Show"
+    __name__ = 'account.landed_cost.show.move'
+
+    move = fields.Many2One('stock.move', "Move", readonly=True)
+    cost = fields.Numeric("Cost", digits=price_digits, readonly=True)
 
 
 class InvoiceLine(metaclass=PoolMeta):
     __name__ = 'account.invoice.line'
-    landed_cost = fields.Many2One('account.landed_cost', 'Landed Cost',
-        readonly=True, select=True,
+    landed_cost = fields.Many2One(
+        'account.landed_cost', "Landed Cost", readonly=True,
         states={
             'invisible': ~Eval('landed_cost'),
             })
