@@ -1,7 +1,7 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from trytond.pool import PoolMeta
 from trytond.model import fields
+from trytond.pool import PoolMeta
 from trytond.pyson import Eval
 
 
@@ -11,7 +11,7 @@ class Template(metaclass=PoolMeta):
         "Landed Cost",
         states={
             'invisible': Eval('type') != 'service',
-            }, depends=['type'])
+            })
 
 
 class Product(metaclass=PoolMeta):
